import re


def is_ip(value):
    if not value is None:
        ipv4 = r"^(?:(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])$"
        ipv6 = r"^((?=.*::)(?!.*::.+::)(::)?([\dA-F]{1,4}:(:|\b)|){5}|([\dA-F]{1,4}:){6})((([\dA-F]{1,4}((?!\3)::|:\b|$))|(?!\2\3)){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})$/i"
        return re.match(ipv4, value) or re.match(ipv6, value)


def get_client_ip_from_x_forwarded_for(value):
    try:

        if not value or value is None:
            return None

        if not isinstance(value, str):
            print("Expected a string, got -" + str(type(value)))
        else:
            # x-forwarded-for may return multiple IP addresses in the format:
            # "client IP, proxy 1 IP, proxy 2 IP"
            # Therefore, the right-most IP address is the IP address of the most recent proxy
            # and the left-most IP address is the IP address of the originating client.
            # source: http://docs.aws.amazon.com/elasticloadbalancing/latest/classic/x-forwarded-headers.html
            # Azure Web App's also adds a port for some reason, so we'll only use the first part (the IP)
            forwardedIps = []

            for e in value.split(','):
                ip = e.strip()
                if ':' in ip:
                    splitted = ip.split(':')
                    if (len(splitted) == 2):
                        forwardedIps.append(splitted[0])
                forwardedIps.append(ip)

            # Sometimes IP addresses in this header can be 'unknown' (http://stackoverflow.com/a/11285650).
            # Therefore taking the left-most IP address that is not unknown
            # A Squid configuration directive can also set the value to "unknown" (http://www.squid-cache.org/Doc/config/forwarded_for/)
            return next(item for item in forwardedIps if is_ip(item))
    except StopIteration:
        return value.encode('utf-8')


def get_client_ip(request):
    try:
        # Standard headers used by Amazon EC2, Heroku, and others.
        if 'HTTP_X_CLIENT_IP' in request.META:
            if is_ip(request.META.get('HTTP_X_CLIENT_IP')):
                return request.META.get('HTTP_X_CLIENT_IP')

        # Load-balancers (AWS ELB) or proxies.
        if 'HTTP_X_FORWARDED_FOR' in request.META:
            x_forwarded_for = get_client_ip_from_x_forwarded_for(request.META.get('HTTP_X_FORWARDED_FOR'))
            if is_ip(x_forwarded_for):
                return x_forwarded_for

        # Cloudflare.
        # @see https://support.cloudflare.com/hc/en-us/articles/200170986-How-does-Cloudflare-handle-HTTP-Request-headers-
        # CF-Connecting-IP - applied to every request to the origin.
        if 'HTTP_CF_CONNECTING_IP' in request.META:
            if is_ip(request.META.get('HTTP_CF_CONNECTING_IP')):
                return request.META.get('HTTP_CF_CONNECTING_IP')

        # Akamai and Cloudflare: True-Client-IP.
        if 'HTTP_TRUE_CLIENT_IP' in request.META:
            if is_ip(request.META.get('HTTP_TRUE_CLIENT_IP')):
                return request.META.get('HTTP_TRUE_CLIENT_IP')

        # Default nginx proxy/fcgi; alternative to x-forwarded-for, used by some proxies.
        if 'HTTP_X_REAL_IP' in request.META:
            if is_ip(request.META.get('HTTP_X_REAL_IP')):
                return request.META.get('HTTP_X_REAL_IP')

        # (Rackspace LB and Riverbed's Stingray)
        # http://www.rackspace.com/knowledge_center/article/controlling-access-to-linux-cloud-sites-based-on-the-client-ip-address
        # https://splash.riverbed.com/docs/DOC-1926
        if 'HTTP_X_CLUSTER_CLIENT_IP' in request.META:
            if is_ip(request.META.get('HTTP_X_CLUSTER_CLIENT_IP')):
                return request.META.get('HTTP_X_CLUSTER_CLIENT_IP')

        if 'HTTP_X_FORWARDED' in request.META:
            if is_ip(request.META.get('HTTP_X_FORWARDED')):
                return request.META.get('HTTP_X_FORWARDED')

        if 'HTTP_FORWARDED_FOR' in request.META:
            if is_ip(request.META.get('HTTP_FORWARDED_FOR')):
                return request.META.get('HTTP_FORWARDED_FOR')

        if 'HTTP_FORWARDED' in request.META:
            if is_ip(request.META.get('HTTP_FORWARDED')):
                return request.META.get('HTTP_FORWARDED')

        return request.META.get('REMOTE_ADDR')
    except:
        return request.META.get('REMOTE_ADDR')



from datetime import datetime
import json
from moesifapi.exceptions.api_exception import *

# Application Configuration
class AppConfig:

    def get_config(self, api_client, debug):
        """Get Config"""
        try:
            config_api_response = api_client.get_app_config()
            return config_api_response
        except APIException as inst:
            if 401 <= inst.response_code <= 403:
                print("Unauthorized access getting application configuration. Please check your Appplication Id.")
            if debug:
                print("Error getting application configuration, with status code:")
                print(inst.response_code)

    def parse_configuration(self, config, debug):
        """Parse configuration object and return Etag, sample rate and last updated time"""
        try:
            return config.headers.get("X-Moesif-Config-ETag"), json.loads(config.raw_body).get('sample_rate', 100), datetime.utcnow()
        except:
            if debug:
                print('Error while parsing the configuration object, setting the sample rate to default')
            return None, 100, datetime.utcnow()

    def get_sampling_percentage(self, config, user_id, company_id):
        """Get sampling percentage"""

        config_body = json.loads(config.raw_body)

        user_sample_rate = config_body.get('user_sample_rate', None)

        company_sample_rate = config_body.get('company_sample_rate', None)

        if user_id and user_sample_rate and user_id in user_sample_rate:
            return user_sample_rate[user_id]

        if company_id and company_sample_rate and company_id in company_sample_rate:
            return company_sample_rate[company_id]

        return config_body.get('sample_rate', 100)



"""
__author__ = 'mpetyx (Michael Petychakis)'
__version__ = "1.0.0"
__maintainer__ = "Michael Petychakis"
__email__ = "michael@orfium.com"
__status__ = "Production"
"""

from __future__ import absolute_import, unicode_literals
from django.conf import settings
from .http_response_catcher import HttpResponseCatcher
from celery import shared_task
from moesifapi.moesif_api_client import MoesifAPIClient
from moesifapi.models import EventModel
from .app_config import AppConfig
from datetime import datetime, timedelta

middleware_settings = settings.MOESIF_MIDDLEWARE
client = MoesifAPIClient(middleware_settings.get('APPLICATION_ID'))
BATCH_SIZE = settings.MOESIF_MIDDLEWARE.get('BATCH_SIZE', 25)
DEBUG = middleware_settings.get('LOCAL_DEBUG', False)

api_client = client.api
response_catcher = HttpResponseCatcher()
api_client.http_call_back = response_catcher

def get_config():
    app_config = AppConfig()
    config = app_config.get_config(api_client, DEBUG)
    sampling_percentage = 100
    config_etag = None
    last_updated_time = datetime.utcnow()
    try:
        if config:
            config_etag, sampling_percentage, last_updated_time = app_config.parse_configuration(config, DEBUG)
    except:
        if DEBUG:
            print('Error while parsing application configuration on initialization')
    return config, config_etag, sampling_percentage, last_updated_time

try:
    get_broker_url = settings.BROKER_URL
    if get_broker_url:
        BROKER_URL = get_broker_url
    else:
        BROKER_URL = None
except AttributeError:
    BROKER_URL = settings.MOESIF_MIDDLEWARE.get('CELERY_BROKER_URL', None)


def queue_get_all(queue_events):
    events = []
    for num_of_events_retrieved in range(0, BATCH_SIZE):
        try:
            if num_of_events_retrieved == BATCH_SIZE:
                break
            events.append(queue_events.get_nowait())
        except:
            break
    return events

@shared_task(ignore_result=True)
def async_client_create_event(moesif_events_queue, config, config_etag, last_updated_time):
    batch_events = []
    try:
        queue_size = moesif_events_queue.qsize()
        while queue_size > 0:
            message = queue_get_all(moesif_events_queue)
            for event in message:
                batch_events.append(EventModel().from_dictionary(event.payload))
                event.ack()
            try:
                queue_size = moesif_events_queue.qsize()
            except ChannelError:
                queue_size = 0
    except ChannelError:
        if DEBUG:
            print("No message to read from the queue")

    if batch_events:
        if DEBUG:
            print("Sending events to Moesif")
        batch_events_api_response = api_client.create_events_batch(batch_events)
        batch_events_response_config_etag = batch_events_api_response.get("X-Moesif-Config-ETag")
        if batch_events_response_config_etag is not None \
                and config_etag is not None \
                and config_etag != batch_events_response_config_etag \
                and datetime.utcnow() > last_updated_time + timedelta(minutes=5):

            try:
                config = config.get_config(api_client, DEBUG)
                config_etag, sampling_percentage, last_updated_time = config.parse_configuration(
                    config, DEBUG)
            except:
                if DEBUG:
                    print('Error while updating the application configuration')
        if DEBUG:
            print("Events sent successfully")
    else:
        if DEBUG:
            print("No events to send")


def exit_handler(moesif_events_queue, scheduler):
    try:
        # Close the close
        moesif_events_queue.close()
        # Shut down the scheduler
        scheduler.shutdown()
    except:
        if DEBUG:
            print("Error while closing the queue or scheduler shut down")

CELERY = False
if settings.MOESIF_MIDDLEWARE.get('USE_CELERY', False):
    if BROKER_URL:
        try:
            from apscheduler.schedulers.background import BackgroundScheduler
            from apscheduler.triggers.interval import IntervalTrigger
            import atexit
            from kombu import Connection
            from kombu.exceptions import ChannelError

            scheduler = BackgroundScheduler(daemon=True)
            scheduler.start()
            config, config_etag, sampling_percentage, last_updated_time = get_config()
            try:
                conn = Connection(BROKER_URL)
                moesif_events_queue = conn.SimpleQueue('moesif_events_queue')
                scheduler.add_job(
                    func=lambda: async_client_create_event(moesif_events_queue, config, config_etag, last_updated_time),
                    trigger=IntervalTrigger(seconds=5),
                    id='moesif_events_batch_job',
                    name='Schedule events batch job every 5 second',
                    replace_existing=True)

                # Exit handler when exiting the app
                atexit.register(lambda: exit_handler(moesif_events_queue, scheduler))
            except:
                if DEBUG:
                    print("Error while connecting to - {0}".format(BROKER_URL))
        except:
            if DEBUG:
                print("Error when scheduling the job")
    else:
        if DEBUG:
            print("Unable to schedule the job as the BROKER_URL is not provided")



from __future__ import print_function

import requests
import threading
import copy
import json
import base64
import re
import django
import logging
import random
import math
from django.conf import settings
from django.utils import timezone
from moesifapi.moesif_api_client import *
from moesifapi.api_helper import *
from moesifapi.exceptions.api_exception import *
from moesifapi.models import *
from django.http import HttpRequest, HttpResponse
from .http_response_catcher import HttpResponseCatcher
from .masks import *
from .client_ip import *
from .update_users import *
from .update_companies import *
from io import BytesIO
from moesifpythonrequest.start_capture.start_capture import StartCapture
from datetime import datetime, timedelta
from .app_config import AppConfig
import uuid

# Logger Config
logging.basicConfig()
logger = logging.getLogger('logger')

CELERY = False
if settings.MOESIF_MIDDLEWARE.get('USE_CELERY', False):
    try:
        import celery
        from .tasks import async_client_create_event
        from kombu import Connection
        try:
            BROKER_URL = settings.BROKER_URL
            if BROKER_URL:
                CELERY = True
            else:
                CELERY = False
        except AttributeError:
            BROKER_URL = settings.MOESIF_MIDDLEWARE.get('CELERY_BROKER_URL', None)
            if BROKER_URL:
                CELERY = True
            else:
                logger.warning("USE_CELERY flag was set to TRUE, but BROKER_URL not found")
                CELERY = False

        try:
            conn = Connection(BROKER_URL)
            simple_queue = conn.SimpleQueue('moesif_events_queue')
        except:
            logger.error("Error while connecting to - {0}".format(BROKER_URL))

    except:
        logger.warning("USE_CELERY flag was set to TRUE, but celery package not found.")
        CELERY = False


class moesif_middleware:
    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.
        self.middleware_settings = settings.MOESIF_MIDDLEWARE
        self.DEBUG = self.middleware_settings.get('LOCAL_DEBUG', False)
        self.LOG_BODY = self.middleware_settings.get('LOG_BODY', True)
        self.client = MoesifAPIClient(self.middleware_settings.get('APPLICATION_ID'))
        # below comment for setting moesif base_uri to a test server.
        if self.middleware_settings.get('LOCAL_DEBUG', False):
            Configuration.BASE_URI = self.middleware_settings.get('LOCAL_MOESIF_BASEURL', 'https://api.moesif.net')
        if settings.MOESIF_MIDDLEWARE.get('CAPTURE_OUTGOING_REQUESTS', False):
            try:
                if self.DEBUG:
                    print('Start capturing outgoing requests')
                # Start capturing outgoing requests
                StartCapture().start_capture_outgoing(settings.MOESIF_MIDDLEWARE)
            except:
                logger.error('Error while starting to capture the outgoing events')
        self.api_version = self.middleware_settings.get('API_VERSION')
        self.api_client = self.client.api
        self.response_catcher = HttpResponseCatcher()
        self.api_client.http_call_back = self.response_catcher
        self.regex_http_ = re.compile(r'^HTTP_.+$')
        self.regex_content_type = re.compile(r'^CONTENT_TYPE$')
        self.regex_content_length = re.compile(r'^CONTENT_LENGTH$')
        self.app_config = AppConfig()
        self.config = self.app_config.get_config(self.api_client, self.DEBUG)
        self.sampling_percentage = 100
        self.config_etag = None
        self.last_updated_time = datetime.utcnow()
        try:
            if self.config:
                self.config_etag, self.sampling_percentage, self.last_updated_time = self.app_config.parse_configuration(self.config, self.DEBUG)
        except:
            if self.DEBUG:
                logger.error('Error while parsing application configuration on initialization')

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        # Initialize Transaction Id
        transaction_id = None

        req_time = timezone.now()

        try:
            request._mo_body = request.body
            request._stream = BytesIO(request.body)
            request._read_started = False
        except:
            request._mo_body = None

        if self.DEBUG:
            print("raw body before getting response")

        response = self.get_response(request)
        # Code to be executed for each request/response after
        # the view is called.

        try:
            skip_event = self.middleware_settings.get('SKIP', None)
            if skip_event is not None:
                if skip_event(request, response):
                    return response
        except:
            if self.DEBUG:
                print("Having difficulty executing skip_event function. Please check moesif settings.")

        req_headers = {}
        regex_http_start = re.compile('^HTTP_')
        try:
            for header in request.META:
                if self.regex_http_.match(header) or self.regex_content_type.match(header) or self.regex_content_length.match(header):
                    normalized_header = regex_http_start.sub('', header)
                    normalized_header = normalized_header.replace('_', '-')
                    req_headers[normalized_header] = request.META[header]
            req_headers = mask_headers(req_headers, self.middleware_settings.get('REQUEST_HEADER_MASKS'))
        except Exception as inst:
            if self.DEBUG:
                print("error encountered while copying request header")
                print(inst)
            req_headers = {}

        if self.DEBUG:
            print("about to print what is in meta %d " % len(request.META))
            for x in request.META:
                print(x, ':', request.META[x])
            print("about to print headers %d " % len(req_headers))
            for x in req_headers:
                print(x, ':', req_headers[x])

        def flatten_to_string(value):
            if type(value) == str:
                return value
            if value is None:
                return ''
            return APIHelper.json_serialize(value)

        req_headers = {k: flatten_to_string(v) for k, v in req_headers.items()}

        # Add transaction id to request headers
        capture_transaction_id = self.middleware_settings.get('DISABLE_TRANSACTION_ID', False)
        if not capture_transaction_id:
            req_trans_id = req_headers.get("X-MOESIF-TRANSACTION-ID", None)
            if req_trans_id:
                transaction_id = req_trans_id
                if not transaction_id:
                    transaction_id = str(uuid.uuid4())
            else:
                transaction_id = str(uuid.uuid4())
            # Add transaction id to the request header
            req_headers["X-MOESIF-TRANSACTION-ID"] = transaction_id

        req_body = None
        req_body_transfer_encoding = None
        if self.LOG_BODY:
            try:
                # print("about to serialize request body" + request.body)
                if self.DEBUG:
                    print("about to process request body")
                if request._mo_body:
                    req_body = json.loads(request._mo_body)
                    req_body = mask_body(req_body, self.middleware_settings.get('REQUEST_BODY_MASKS'))
            except:
                if request._mo_body:
                    req_body = base64.standard_b64encode(request._mo_body).decode(encoding="UTF-8")
                    req_body_transfer_encoding = 'base64'

        ip_address = get_client_ip(request)
        uri = request.scheme + "://" + request.get_host() + request.get_full_path()

        def mapper(key):
            return copy.deepcopy(response[key])

        # Add transaction id to request headers
        if transaction_id:
            response._headers["x-moesif-transaction-id"] = ("X-Moesif-Transaction-Id", transaction_id)

        # a little hacky, using _headers, which is intended as a private variable.
        rsp_headers = {k: mapper(k) for k, v in response._headers.items()}

        rsp_headers = mask_headers(rsp_headers, self.middleware_settings.get('RESPONSE_HEADER_MASKS'))

        rsp_body = None
        rsp_body_transfer_encoding = None
        if self.LOG_BODY and isinstance(response, HttpResponse) and response.content:
            if self.DEBUG:
                print("about to process response")
                print(response.content)
            try:
                rsp_body = json.loads(response.content)
                if self.DEBUG:
                    print("json parsed succesfully")
                rsp_body = mask_body(rsp_body, self.middleware_settings.get('RESPONSE_BODY_MASKS'))

            except:
                if self.DEBUG:
                    print("could not json parse, so base64 encode")
                rsp_body = base64.standard_b64encode(response.content).decode(encoding="UTF-8")
                rsp_body_transfer_encoding = 'base64'
                if self.DEBUG:
                    print("base64 encoded body: " + str(rsp_body))

        rsp_time = timezone.now()

        event_req = EventRequestModel(time=req_time.isoformat(),
                                      uri=uri,
                                      verb=request.method,
                                      api_version=self.api_version,
                                      ip_address=ip_address,
                                      headers=req_headers,
                                      body=req_body,
                                      transfer_encoding=req_body_transfer_encoding)

        event_rsp = EventResponseModel(time=rsp_time.isoformat(),
                                       status=response.status_code,
                                       headers=rsp_headers,
                                       body=rsp_body,
                                       transfer_encoding=rsp_body_transfer_encoding)

        username = None
        try:
            username = request.user.username
        except:
            username = None

        try:
            identify_user = self.middleware_settings.get('IDENTIFY_USER', None)
            if identify_user is not None:
                username = identify_user(request, response)
        except:
            if self.DEBUG:
                print("can not execute identify_user function, please check moesif settings.")

        company_id = None
        try:
            identify_company = self.middleware_settings.get('IDENTIFY_COMPANY', None)
            if identify_company is not None:
                company_id = identify_company(request, response)
        except:
            if self.DEBUG:
                print("can not execute identify_company function, please check moesif settings.")

        metadata = None
        try:
            get_metadata = self.middleware_settings.get('GET_METADATA', None)
            if get_metadata is not None:
                metadata = get_metadata(request, response)
        except:
            if self.DEBUG:
                print("can not execute get_metadata function, please check moesif settings.")

        session_token = None
        try:
            session_token = request.session.session_key
        except:
            session_token = None

        try:
            get_session_token = self.middleware_settings.get('GET_SESSION_TOKEN', None)
            if get_session_token is not None:
                session_token = get_session_token(request, response)
        except:
            if self.DEBUG:
                print("Can not execute get_session_token function. Please check moesif settings.")

        event_model = EventModel(request=event_req,
                                 response=event_rsp,
                                 user_id=username,
                                 company_id=company_id,
                                 session_token=session_token,
                                 metadata=metadata,
                                 direction="Incoming")

        try:
            mask_event_model = self.middleware_settings.get('MASK_EVENT_MODEL', None)
            if mask_event_model is not None:
                event_model = mask_event_model(event_model)
        except:
            if self.DEBUG:
                print("Can not execute MASK_EVENT_MODEL function. Please check moesif settings.")

        def sending_event():
            if self.DEBUG:
                print("sending event to moesif")
            try:
                if not CELERY:
                    event_api_response = self.api_client.create_event(event_model)
                    event_response_config_etag = event_api_response.get("X-Moesif-Config-ETag")
                    if event_response_config_etag is not None \
                            and self.config_etag is not None \
                            and self.config_etag != event_response_config_etag \
                            and datetime.utcnow() > self.last_updated_time + timedelta(minutes=5):
                        try:
                            self.config = self.app_config.get_config(self.api_client, self.DEBUG)
                            self.config_etag, self.sampling_percentage, self.last_updated_time = self.app_config.parse_configuration(self.config, self.DEBUG)
                        except:
                            if self.DEBUG:
                                logger.error('Error while updating the application configuration')
                else:
                    try:
                        message = event_model.to_dictionary()
                        simple_queue.put(message)
                        if self.DEBUG:
                            print("Event added to the queue")
                    except:
                        if self.DEBUG:
                            print("Error while adding event to the queue")
                if self.DEBUG:
                    print("Event sent successfully")
            except APIException as inst:
                if 401 <= inst.response_code <= 403:
                    print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                if self.DEBUG:
                    print("Error sending event to Moesif, with status code:")
                    print(inst.response_code)

        random_percentage = random.random() * 100

        self.sampling_percentage = self.app_config.get_sampling_percentage(self.config, username, company_id)
        if self.sampling_percentage >= random_percentage:
            event_model.weight = 1 if self.sampling_percentage == 0 else math.floor(100 / self.sampling_percentage)
            if CELERY:
                sending_event()
            else:
                # send the event to moesif via background so not blocking
                sending_background_thread = threading.Thread(target=sending_event)
                sending_background_thread.start()

        return response

    def update_user(self, user_profile):
        update_user(user_profile, self.api_client, self.DEBUG)

    def update_users_batch(self, user_profiles):
        update_users_batch(user_profiles, self.api_client, self.DEBUG)

    def update_company(self, company_profile):
        update_company(company_profile, self.api_client, self.DEBUG)

    def update_companies_batch(self, companies_profiles):
        update_companies_batch(companies_profiles, self.api_client, self.DEBUG)



from moesifapi.models import *
from moesifapi.exceptions.api_exception import *
from moesifapi.api_helper import *


def update_user(user_profile, api_client, DEBUG):
    if not user_profile:
        print('Expecting the input to be either of the type - UserModel, dict or json while updating user')
    else:
        if isinstance(user_profile, dict):
            if 'user_id' in user_profile:
                try:
                    api_client.update_user(UserModel.from_dictionary(user_profile))
                    if DEBUG:
                        print('User Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating user, with status code:")
                        print(inst.response_code)
            else:
                print('To update an user, an user_id field is required')

        elif isinstance(user_profile, UserModel):
            if user_profile.user_id is not None:
                try:
                    api_client.update_user(user_profile)
                    if DEBUG:
                        print('User Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating user, with status code:")
                        print(inst.response_code)
            else:
                print('To update an user, an user_id field is required')
        else:
            try:
                user_profile_json = APIHelper.json_deserialize(user_profile)
                if 'user_id' in user_profile_json:
                    try:
                        api_client.update_user(UserModel.from_dictionary(user_profile_json))
                        if DEBUG:
                            print('User Profile updated successfully')
                    except APIException as inst:
                        if 401 <= inst.response_code <= 403:
                            print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                        if DEBUG:
                            print("Error while updating user, with status code:")
                            print(inst.response_code)
                else:
                    print('To update an user, an user_id field is required')
            except:
                print('Error while deserializing the json, please make sure the json is valid')


def update_users_batch(user_profiles, api_client, DEBUG):
    if not user_profiles:
        print('Expecting the input to be either of the type - List of UserModel, dict or json while updating users')
    else:
        if all(isinstance(user, dict) for user in user_profiles):
            if all('user_id' in user for user in user_profiles):
                try:
                    batch_profiles = [UserModel.from_dictionary(d) for d in user_profiles]
                    api_client.update_users_batch(batch_profiles)
                    if DEBUG:
                        print('Users Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating users, with status code:")
                        print(inst.response_code)
            else:
                print('To update users, an user_id field is required')

        elif all(isinstance(user, UserModel) for user in user_profiles):
            if all(user.user_id is not None for user in user_profiles):
                try:
                    api_client.update_users_batch(user_profiles)
                    if DEBUG:
                        print('Users Profile updated successfully')
                except APIException as inst:
                    if 401 <= inst.response_code <= 403:
                        print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                    if DEBUG:
                        print("Error while updating users, with status code:")
                        print(inst.response_code)
            else:
                print('To update users, an user_id field is required')
        else:
            try:
                user_profiles_json = [APIHelper.json_deserialize(d) for d in user_profiles]
                if all(isinstance(user, dict) for user in user_profiles_json) and all(
                                'user_id' in user for user in user_profiles_json):
                    try:
                        batch_profiles = [UserModel.from_dictionary(d) for d in user_profiles_json]
                        api_client.update_users_batch(batch_profiles)
                        if DEBUG:
                            print('Users Profile updated successfully')
                    except APIException as inst:
                        if 401 <= inst.response_code <= 403:
                            print("Unauthorized access sending event to Moesif. Please check your Appplication Id.")
                        if DEBUG:
                            print("Error while updating users, with status code:")
                            print(inst.response_code)
                else:
                    print('To update users, an user_id field is required')
            except:
                print('Error while deserializing the json, please make sure the json is valid')